@@ -31,16 +31,13 @@
 #include <QSortFilterProxyModel>
 #include <QVector>
 #include <QTimer>
-#include <QLineEdit>
 
 #include <KLocalizedString>
-#include <KMessageBox>
 
 Q_DECLARE_METATYPE(GpgME::Key)
 
 namespace
 {
-
 class ProxyModel : public QSortFilterProxyModel
 {
     Q_OBJECT
@@ -63,6 +60,44 @@ public:
         qDeleteAll(mBackItems);
     }
 
+    bool lessThan(const QModelIndex &left, const QModelIndex &right) const override
+    {
+        const auto leftKey =  sourceModel()->data(left, Kleo::KeyListModelInterface::KeyRole).value<GpgME::Key>();
+        const auto rightKey = sourceModel()->data(right, Kleo::KeyListModelInterface::KeyRole).value<GpgME::Key>();
+        if (leftKey.isNull()) {
+            return false;
+        }
+        if (rightKey.isNull()) {
+            return true;
+        }
+        // As we display UID(0) this is ok. We probably need a get Best UID at some point.
+        const auto lUid = leftKey.userID(0);
+        const auto rUid = rightKey.userID(0);
+        if (lUid.validity() == rUid.validity()) {
+            /* Both are the same check which one is newer. */
+            time_t oldTime = 0;
+            for (const GpgME::Subkey s: leftKey.subkeys()) {
+                if (s.isRevoked() || s.isInvalid() || s.isDisabled()) {
+                    continue;
+                }
+                if (s.creationTime() > oldTime) {
+                    oldTime= s.creationTime();
+                }
+            }
+            time_t newTime = 0;
+            for (const GpgME::Subkey s: rightKey.subkeys()) {
+                if (s.isRevoked() || s.isInvalid() || s.isDisabled()) {
+                    continue;
+                }
+                if (s.creationTime() > newTime) {
+                    newTime = s.creationTime();
+                }
+            }
+            return newTime < oldTime;
+        }
+        return lUid.validity() > rUid.validity();
+    }
+
     bool isCustomItem(const int row) const
     {
         return row < mFrontItems.count() || row >= mFrontItems.count() + QSortFilterProxyModel::rowCount();
@@ -203,8 +238,6 @@ public:
         default:
             return QSortFilterProxyModel::data(index, role);
         }
-
-        return QVariant();
     }
 
 private:
@@ -226,12 +259,13 @@ public:
     {
     }
 
-    Kleo::AbstractKeyListModel *model;
-    Kleo::KeyListSortFilterProxyModel *sortFilterProxy;
-    ProxyModel *proxyModel;
+    Kleo::AbstractKeyListModel *model = nullptr;
+    Kleo::KeyListSortFilterProxyModel *sortFilterProxy = nullptr;
+    ProxyModel *proxyModel = nullptr;
     std::shared_ptr<Kleo::KeyCache> cache;
     QString defaultKey;
-    bool wasEnabled;
+    bool wasEnabled = false;
+    bool secretOnly;
 
 private:
     KeySelectionCombo * const q;
@@ -241,12 +275,16 @@ private:
 
 using namespace Kleo;
 
-
 KeySelectionCombo::KeySelectionCombo(QWidget* parent)
+    : KeySelectionCombo(true, parent)
+{}
+
+KeySelectionCombo::KeySelectionCombo(bool secretOnly, QWidget* parent)
     : QComboBox(parent)
     , d(new KeySelectionComboPrivate(this))
 {
     d->model = Kleo::AbstractKeyListModel::createFlatKeyListModel(this);
+    d->secretOnly = secretOnly;
 
     d->sortFilterProxy = new Kleo::KeyListSortFilterProxyModel(this);
     d->sortFilterProxy->setSourceModel(d->model);
@@ -255,7 +293,16 @@ KeySelectionCombo::KeySelectionCombo(QWidget* parent)
     d->proxyModel->setSourceModel(d->sortFilterProxy);
 
     setModel(d->proxyModel);
-    connect(this, static_cast<void(KeySelectionCombo::*)(int)>(&KeySelectionCombo::currentIndexChanged), this, [this](int row) { if (row >= 0 && row < d->proxyModel->rowCount()) { if (d->proxyModel->isCustomItem(row)) { Q_EMIT customItemSelected(d->proxyModel->index(row, 0).data(Qt::UserRole)); } else { Q_EMIT currentKeyChanged(currentKey()); } } });
+    connect(this, static_cast<void(KeySelectionCombo::*)(int)>(&KeySelectionCombo::currentIndexChanged),
+            this, [this](int row) {
+                if (row >= 0 && row < d->proxyModel->rowCount()) {
+                    if (d->proxyModel->isCustomItem(row)) {
+                        Q_EMIT customItemSelected(d->proxyModel->index(row, 0).data(Qt::UserRole));
+                    } else {
+                        Q_EMIT currentKeyChanged(currentKey());
+                    }
+                }
+            });
 
     d->cache = Kleo::KeyCache::mutableInstance();
 
@@ -273,8 +320,7 @@ void KeySelectionCombo::init()
             this, [this]() {
                     // Set useKeyCache ensures that the cache is populated
                     // so this can be a blocking call if the cache is not initalized 
-                    qDebug() << "Key listing done"; 
-                    d->model->useKeyCache(true, true);
+                    d->model->useKeyCache(true, d->secretOnly);
                     d->proxyModel->removeCustomItem(QStringLiteral("-libkleo-loading-keys"));
                     setEnabled(d->wasEnabled);
                     Q_EMIT keyListingFinished(); 
@@ -285,7 +331,7 @@ void KeySelectionCombo::init()
     if (!d->cache->initialized()) {
         refreshKeys();
     } else {
-        d->model->useKeyCache(true, true);
+        d->model->useKeyCache(true, d->secretOnly);
         Q_EMIT keyListingFinished();
     }
 }
@@ -294,6 +340,7 @@ void KeySelectionCombo::init()
 void KeySelectionCombo::setKeyFilter(const std::shared_ptr<const KeyFilter> &kf)
 {
     d->sortFilterProxy->setKeyFilter(kf);
+    d->proxyModel->sort(0);
     setCurrentKey(d->defaultKey);
 }
 
@@ -363,6 +410,7 @@ void KeySelectionCombo::prependCustomItem(const QIcon &icon, const QString &text
 void Kleo::KeySelectionCombo::setDefaultKey(const QString &fingerprint)
 {
     d->defaultKey = fingerprint;
+    setCurrentKey(d->defaultKey);
 }
 
 QString Kleo::KeySelectionCombo::defaultKey() const
